@@ -76,6 +76,7 @@
 #define V4L2_BUFFERS_REQUESTED_BY_READ	0x0200
 #define V4L2_STREAM_CONTROLLED_BY_READ	0x0400
 #define V4L2_SUPPORTS_READ		0x0800
+#define V4L2_IS_UVC			0x1000
 
 #define V4L2_MMAP_OFFSET_MAGIC      0xABCDEF00u
 
@@ -264,9 +265,11 @@ static int v4l2_dequeue_and_convert(int index, struct v4l2_buffer *buf,
 
   do {
     if ((result = syscall(SYS_ioctl, devices[index].fd, VIDIOC_DQBUF, buf))) {
-      int saved_err = errno;
-      V4L2_LOG_ERR("dequeuing buf: %s\n", strerror(errno));
-      errno = saved_err;
+      if (errno != EAGAIN) {
+	int saved_err = errno;
+	V4L2_LOG_ERR("dequeuing buf: %s\n", strerror(errno));
+	errno = saved_err;
+      }
       return result;
     }
 
@@ -490,6 +493,8 @@ int v4l2_fd_open(int fd, int v4l2_flags)
   devices[index].flags = v4l2_flags;
   if (cap.capabilities & V4L2_CAP_READWRITE)
     devices[index].flags |= V4L2_SUPPORTS_READ;
+  if (!strcmp((char *)cap.driver, "uvcvideo"))
+    devices[index].flags |= V4L2_IS_UVC;
   devices[index].open_count = 1;
   devices[index].src_fmt = fmt;
   devices[index].dest_fmt = fmt;
@@ -755,6 +760,9 @@ int v4l2_ioctl (int fd, unsigned long int request, ...)
 
     case VIDIOC_ENUM_FRAMEINTERVALS:
       result = v4lconvert_enum_frameintervals(devices[index].convert, arg);
+      if (result)
+        V4L2_LOG("ENUM_FRAMEINTERVALS Error: %s", 
+          v4lconvert_get_error_message(devices[index].convert));
       break;
 
     case VIDIOC_TRY_FMT:
@@ -766,7 +774,10 @@ int v4l2_ioctl (int fd, unsigned long int request, ...)
 	struct v4l2_format src_fmt, *dest_fmt = arg;
 	struct v4l2_pix_format req_pix_fmt;
 
-	if (v4l2_pix_fmt_identical(&devices[index].dest_fmt, dest_fmt)) {
+	/* Don't be lazy on uvc cams, as this triggers a bug in the uvcvideo
+	   driver in kernel <= 2.6.28 (with certain cams) */
+	if (!(devices[index].flags & V4L2_IS_UVC) &&
+	    v4l2_pix_fmt_identical(&devices[index].dest_fmt, dest_fmt)) {
 	  *dest_fmt = devices[index].dest_fmt;
 	  result = 0;
 	  break;
@@ -791,8 +802,12 @@ int v4l2_ioctl (int fd, unsigned long int request, ...)
 					 &src_fmt);
 	}
 
-	if (result)
+	if (result) {
+	  saved_err = errno;
+	  V4L2_LOG("S_FMT error trying format: %s\n", strerror(errno));
+	  errno = saved_err;
 	  break;
+	}
 
 	if (src_fmt.fmt.pix.pixelformat != dest_fmt->fmt.pix.pixelformat &&
 	    v4l2_log_file) {
@@ -807,7 +822,8 @@ int v4l2_ioctl (int fd, unsigned long int request, ...)
 
 	/* Maybe after try format has adjusted width/height etc, to whats
 	   available nothing has changed (on the cam side) ? */
-	if (v4l2_pix_fmt_identical(&devices[index].src_fmt, &src_fmt)) {
+	if (!(devices[index].flags & V4L2_IS_UVC) &&
+	    v4l2_pix_fmt_identical(&devices[index].src_fmt, &src_fmt)) {
 	  v4l2_set_src_and_dest_format(index, &devices[index].src_fmt,
 				       dest_fmt);
 	  result = 0;
@@ -936,7 +952,7 @@ int v4l2_ioctl (int fd, unsigned long int request, ...)
 	   but we need the buffer _now_ to write our converted data
 	   to it! */
 	if (devices[index].convert_mmap_buf == MAP_FAILED) {
-	  devices[index].convert_mmap_buf = (void *)syscall(SYS_mmap2,
+	  devices[index].convert_mmap_buf = (void *)syscall(SYS_mmap2, NULL,
 						   (size_t)(
 						     devices[index].no_frames *
 						     V4L2_FRAME_BUF_SIZE),
