@@ -32,10 +32,11 @@
 #define SUPPORTED_DST_PIXFMTS \
   { V4L2_PIX_FMT_RGB24,  0 }, \
   { V4L2_PIX_FMT_BGR24,  0 }, \
-  { V4L2_PIX_FMT_YUV420, 0 }
+  { V4L2_PIX_FMT_YUV420, 0 }, \
+  { V4L2_PIX_FMT_YVU420, 0 }
 
 static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
-  unsigned int pixelformat);
+  unsigned int pixelformat, int index);
 
 /* Note uncompressed formats must go first so that they are prefered by
    v4lconvert_try_format for low resolutions */
@@ -107,10 +108,9 @@ struct v4lconvert_data *v4lconvert_create(int fd)
     for (j = 0; j < ARRAY_SIZE(supported_src_pixfmts); j++)
       if (fmt.pixelformat == supported_src_pixfmts[j].fmt) {
 	data->supported_src_formats |= 1 << j;
+	v4lconvert_get_framesizes(data, fmt.pixelformat, j);
 	break;
       }
-
-    v4lconvert_get_framesizes(data, fmt.pixelformat);
   }
 
   data->no_formats = i;
@@ -122,6 +122,8 @@ struct v4lconvert_data *v4lconvert_create(int fd)
 	data->flags = v4lconvert_flags[i].flags;
 	break;
       }
+    if (!strcmp((char *)cap.driver, "uvcvideo"))
+      data->flags |= V4LCONVERT_IS_UVC;
   }
 
   return data;
@@ -186,6 +188,64 @@ int v4lconvert_enum_fmt(struct v4lconvert_data *data, struct v4l2_fmtdesc *fmt)
   return 0;
 }
 
+/* Find out what format to use based on the (cached) results of enum
+   framesizes instead of doing a zillion try_fmt calls. This function
+   currently is intended for use with UVC cams only. This is esp.
+   important for UVC based cams as doing try_fmt there actually causes I/O */
+static int v4lconvert_do_try_format_uvc(struct v4lconvert_data *data,
+  struct v4l2_format *dest_fmt, struct v4l2_format *src_fmt)
+{
+  int i;
+  unsigned int closest_fmt_size_diff = -1;
+  int best_framesize = 0;/* Just use the first format if no small enough one */
+  int best_format = 0;
+
+  for (i = 0; i < data->no_framesizes; i++) {
+    if (data->framesizes[i].discrete.width <= dest_fmt->fmt.pix.width &&
+	data->framesizes[i].discrete.height <= dest_fmt->fmt.pix.height) {
+      int size_x_diff = dest_fmt->fmt.pix.width -
+			data->framesizes[i].discrete.width;
+      int size_y_diff = dest_fmt->fmt.pix.height -
+			data->framesizes[i].discrete.height;
+      unsigned int size_diff = size_x_diff * size_x_diff +
+			       size_y_diff * size_y_diff;
+
+      if (size_diff < closest_fmt_size_diff) {
+        closest_fmt_size_diff = size_diff;
+	best_framesize = i;
+      }
+    }
+  }
+
+  for (i = 0; i < ARRAY_SIZE(supported_src_pixfmts); i++) {
+    /* is this format supported? */
+    if (!(data->framesizes[best_framesize].pixel_format & (1 << i)))
+      continue;
+
+    if (!best_format ||
+	supported_src_pixfmts[i].fmt == dest_fmt->fmt.pix.pixelformat ||
+	((data->framesizes[best_framesize].discrete.width > 180 ||
+	  data->framesizes[best_framesize].discrete.height > 148) &&
+	 (supported_src_pixfmts[i].flags & V4LCONVERT_COMPRESSED)))
+      best_format = supported_src_pixfmts[i].fmt;
+  }
+
+  dest_fmt->fmt.pix.width = data->framesizes[best_framesize].discrete.width;
+  dest_fmt->fmt.pix.height = data->framesizes[best_framesize].discrete.height;
+  dest_fmt->fmt.pix.field = V4L2_FIELD_NONE; /* UVC has no fields */
+  /* Not pretty, the pwc driver doesn't fill these in try_fmt either though,
+     so we should be able to get away with this. */
+  dest_fmt->fmt.pix.bytesperline = 0;
+  dest_fmt->fmt.pix.sizeimage = 0;
+  dest_fmt->fmt.pix.colorspace = 0;
+  dest_fmt->fmt.pix.priv = 0;
+
+  *src_fmt = *dest_fmt;
+  src_fmt->fmt.pix.pixelformat = best_format;
+
+  return 0;
+}
+
 static int v4lconvert_do_try_format(struct v4lconvert_data *data,
   struct v4l2_format *dest_fmt, struct v4l2_format *src_fmt)
 {
@@ -194,6 +254,9 @@ static int v4lconvert_do_try_format(struct v4lconvert_data *data,
   unsigned int desired_pixfmt = dest_fmt->fmt.pix.pixelformat;
   struct v4l2_format try_fmt, closest_fmt = { .type = 0 };
 
+  if (data->flags & V4LCONVERT_IS_UVC)
+    return v4lconvert_do_try_format_uvc(data, dest_fmt, src_fmt);
+
   for (i = 0; i < ARRAY_SIZE(supported_src_pixfmts); i++) {
     /* is this format supported? */
     if (!(data->supported_src_formats & (1 << i)))
@@ -243,6 +306,7 @@ static void v4lconvert_fixup_fmt(struct v4l2_format *fmt)
     fmt->fmt.pix.sizeimage = fmt->fmt.pix.width * fmt->fmt.pix.height * 3;
     break;
   case V4L2_PIX_FMT_YUV420:
+  case V4L2_PIX_FMT_YVU420:
     fmt->fmt.pix.bytesperline = fmt->fmt.pix.width;
     fmt->fmt.pix.sizeimage = fmt->fmt.pix.width * fmt->fmt.pix.height * 3 / 2;
     break;
@@ -268,29 +332,30 @@ int v4lconvert_try_format(struct v4lconvert_data *data,
     return result;
   }
 
-  /* In case of a non exact resolution match, see if this is a resolution we
-     can support by cropping a slightly larger resolution to give the app
-     exactly what it asked for */
+  /* In case of a non exact resolution match, see if this is a well known
+     resolution some apps are hardcoded too and try to give the app what it
+     asked for by cropping a slightly larger resolution */
   if (try_dest.fmt.pix.width != desired_width ||
       try_dest.fmt.pix.height != desired_height) {
     for (i = 0; i < ARRAY_SIZE(v4lconvert_crop_res); i++) {
       if (v4lconvert_crop_res[i][0] == desired_width &&
 	  v4lconvert_crop_res[i][1] == desired_height) {
-	struct v4l2_format try2_dest, try2_src;
+	struct v4l2_format try2_src, try2_dest = *dest_fmt;
+
 	/* Note these are chosen so that cropping to vga res just works for
 	   vv6410 sensor cams, which have 356x292 and 180x148 */
-	unsigned int max_width = desired_width * 113 / 100;
-	unsigned int max_height = desired_height * 124 / 100;
-
-	try2_dest = *dest_fmt;
-	try2_dest.fmt.pix.width = max_width;
-	try2_dest.fmt.pix.height = max_height;
+	try2_dest.fmt.pix.width = desired_width * 113 / 100;
+	try2_dest.fmt.pix.height = desired_height * 124 / 100;
 	result = v4lconvert_do_try_format(data, &try2_dest, &try2_src);
 	if (result == 0 &&
-	    try2_dest.fmt.pix.width >= desired_width &&
-	    try2_dest.fmt.pix.width <= max_width &&
-	    try2_dest.fmt.pix.height >= desired_height &&
-	    try2_dest.fmt.pix.height <= max_height) {
+	    ((try2_dest.fmt.pix.width >= desired_width &&
+	      try2_dest.fmt.pix.width <= desired_width * 5 / 4 &&
+	      try2_dest.fmt.pix.height >= desired_height &&
+	      try2_dest.fmt.pix.height <= desired_height * 5 / 4) ||
+	     (try2_dest.fmt.pix.width >= desired_width * 2 &&
+	      try2_dest.fmt.pix.width <= desired_width * 5 / 2 &&
+	      try2_dest.fmt.pix.height >= desired_height &&
+	      try2_dest.fmt.pix.height <= desired_height * 5 / 2))) {
 	  /* Success! */
 	  try2_dest.fmt.pix.width = desired_width;
 	  try2_dest.fmt.pix.height = desired_height;
@@ -353,13 +418,15 @@ static unsigned char *v4lconvert_alloc_buffer(struct v4lconvert_data *data,
 }
 
 static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
-  unsigned int src_pix_fmt, unsigned int dest_pix_fmt,
-  unsigned int width, unsigned int height,
-  unsigned char *src, int src_size, unsigned char *dest)
+  unsigned char *src, int src_size, unsigned char *dest,
+  const struct v4l2_format *src_fmt, unsigned int dest_pix_fmt)
 {
   unsigned int header_width, header_height;
   int result = 0, jpeg_flags = TINYJPEG_FLAGS_MJPEG_TABLE;
   unsigned char *components[3];
+  unsigned int src_pix_fmt = src_fmt->fmt.pix.pixelformat;
+  unsigned int width  = src_fmt->fmt.pix.width;
+  unsigned int height = src_fmt->fmt.pix.height;
 
   switch (src_pix_fmt) {
     case V4L2_PIX_FMT_PJPG:
@@ -388,7 +455,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	/* Check for (pixart) rotated JPEG */
 	if (header_width == height && header_height == width) {
 	  if (!(data->flags & V4LCONVERT_ROTATE_90)) {
-	    V4LCONVERT_ERR("JPEG needs 90 degree rotation");
+	    V4LCONVERT_ERR("JPEG needs 90 degree rotation\n");
 	    data->flags |= V4LCONVERT_ROTATE_90;
 	    errno = EAGAIN;
 	    return -1;
@@ -403,8 +470,6 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       }
 
       components[0] = dest;
-      components[1] = components[0] + width * height;
-      components[2] = components[1] + width * height / 4;
 
       switch (dest_pix_fmt) {
       case V4L2_PIX_FMT_RGB24:
@@ -416,6 +481,14 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_BGR24);
 	break;
       case V4L2_PIX_FMT_YUV420:
+	components[1] = components[0] + width * height;
+	components[2] = components[1] + width * height / 4;
+	tinyjpeg_set_components(data->jdec, components, 3);
+	result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_YUV420P);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	components[2] = components[0] + width * height;
+	components[1] = components[2] + width * height / 4;
 	tinyjpeg_set_components(data->jdec, components, 3);
 	result = tinyjpeg_decode(data->jdec, TINYJPEG_FMT_YUV420P);
 	break;
@@ -427,7 +500,7 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	   the upper layer this is an intermediate fault and it should try
 	   again with a new buffer by setting errno to EAGAIN */
 	if (src_pix_fmt == V4L2_PIX_FMT_PJPG) {
-	  V4LCONVERT_ERR("Error decompressing JPEG: %s",
+	  V4LCONVERT_ERR("decompressing JPEG: %s",
 	    tinyjpeg_get_errorstring(data->jdec));
 	  errno = EAGAIN;
 	  return -1;
@@ -453,7 +526,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_bayer_to_bgr24(src, dest, width, height, src_pix_fmt);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt);
+	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_bayer_to_yuv420(src, dest, width, height, src_pix_fmt, 1);
 	break;
       }
       break;
@@ -464,35 +540,40 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
     case V4L2_PIX_FMT_SPCA508:
     {
       unsigned char *d;
+      int yvu = 0;
 
-      if (dest_pix_fmt != V4L2_PIX_FMT_YUV420) {
-        d = v4lconvert_alloc_buffer(data, width * height * 3 / 2,
-              &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
-        if (!d)
-          return -1;
+      if (dest_pix_fmt != V4L2_PIX_FMT_YUV420 &&
+	  dest_pix_fmt != V4L2_PIX_FMT_YVU420) {
+	d = v4lconvert_alloc_buffer(data, width * height * 3 / 2,
+	      &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
+	if (!d)
+	  return -1;
       } else
-        d = dest;
+	d = dest;
+
+      if (dest_pix_fmt == V4L2_PIX_FMT_YVU420)
+	yvu = 1;
 
       switch (src_pix_fmt) {
 	case V4L2_PIX_FMT_SPCA501:
-	  v4lconvert_spca501_to_yuv420(src, d, width, height);
+	  v4lconvert_spca501_to_yuv420(src, d, width, height, yvu);
 	  break;
 	case V4L2_PIX_FMT_SPCA505:
-	  v4lconvert_spca505_to_yuv420(src, d, width, height);
+	  v4lconvert_spca505_to_yuv420(src, d, width, height, yvu);
 	  break;
 	case V4L2_PIX_FMT_SPCA508:
-	  v4lconvert_spca508_to_yuv420(src, d, width, height);
+	  v4lconvert_spca508_to_yuv420(src, d, width, height, yvu);
 	  break;
       }
 
       switch (dest_pix_fmt) {
       case V4L2_PIX_FMT_RGB24:
 	v4lconvert_yuv420_to_rgb24(data->convert_pixfmt_buf, dest, width,
-	                           height);
+				   height, yvu);
 	break;
       case V4L2_PIX_FMT_BGR24:
 	v4lconvert_yuv420_to_bgr24(data->convert_pixfmt_buf, dest, width,
-	                           height);
+				   height, yvu);
 	break;
       }
       break;
@@ -507,9 +588,9 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       unsigned int bayer_fmt = 0;
 
       tmpbuf = v4lconvert_alloc_buffer(data, width * height,
-            &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
+	    &data->convert_pixfmt_buf, &data->convert_pixfmt_buf_size);
       if (!tmpbuf)
-        return -1;
+	return -1;
 
       switch (src_pix_fmt) {
 	case V4L2_PIX_FMT_SPCA561:
@@ -534,7 +615,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_bayer_to_bgr24(tmpbuf, dest, width, height, bayer_fmt);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_bayer_to_yuv420(tmpbuf, dest, width, height, bayer_fmt);
+	v4lconvert_bayer_to_yuv420(tmpbuf, dest, width, height, bayer_fmt, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_bayer_to_yuv420(tmpbuf, dest, width, height, bayer_fmt, 1);
 	break;
       }
       break;
@@ -546,7 +630,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_swap_rgb(src, dest, width, height);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	printf("FIXME add rgb24 -> yuv420 conversion\n");
+	v4lconvert_rgb24_to_yuv420(src, dest, src_fmt, 0, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_rgb24_to_yuv420(src, dest, src_fmt, 0, 1);
 	break;
       }
       break;
@@ -557,7 +644,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_swap_rgb(src, dest, width, height);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	printf("FIXME add bgr24 -> yuv420 conversion\n");
+	v4lconvert_rgb24_to_yuv420(src, dest, src_fmt, 1, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_rgb24_to_yuv420(src, dest, src_fmt, 1, 1);
 	break;
       }
       break;
@@ -566,11 +656,30 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
       switch (dest_pix_fmt) {
       case V4L2_PIX_FMT_RGB24:
 	v4lconvert_yuv420_to_rgb24(src, dest, width,
-				   height);
+				   height, 0);
 	break;
       case V4L2_PIX_FMT_BGR24:
 	v4lconvert_yuv420_to_bgr24(src, dest, width,
-				   height);
+				   height, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_swap_uv(src, dest, src_fmt);
+	break;
+      }
+      break;
+
+    case V4L2_PIX_FMT_YVU420:
+      switch (dest_pix_fmt) {
+      case V4L2_PIX_FMT_RGB24:
+	v4lconvert_yuv420_to_rgb24(src, dest, width,
+				   height, 1);
+	break;
+      case V4L2_PIX_FMT_BGR24:
+	v4lconvert_yuv420_to_bgr24(src, dest, width,
+				   height, 1);
+	break;
+      case V4L2_PIX_FMT_YUV420:
+	v4lconvert_swap_uv(src, dest, src_fmt);
 	break;
       }
       break;
@@ -584,7 +693,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_yuyv_to_bgr24(src, dest, width, height);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_yuyv_to_yuv420(src, dest, width, height);
+	v4lconvert_yuyv_to_yuv420(src, dest, width, height, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_yuyv_to_yuv420(src, dest, width, height, 1);
 	break;
       }
       break;
@@ -598,7 +710,10 @@ static int v4lconvert_convert_pixfmt(struct v4lconvert_data *data,
 	v4lconvert_yvyu_to_bgr24(src, dest, width, height);
 	break;
       case V4L2_PIX_FMT_YUV420:
-	v4lconvert_yvyu_to_yuv420(src, dest, width, height);
+	v4lconvert_yvyu_to_yuv420(src, dest, width, height, 0);
+	break;
+      case V4L2_PIX_FMT_YVU420:
+	v4lconvert_yvyu_to_yuv420(src, dest, width, height, 1);
 	break;
       }
       break;
@@ -644,6 +759,7 @@ int v4lconvert_convert(struct v4lconvert_data *data,
       temp_needed = my_src_fmt.fmt.pix.width * my_src_fmt.fmt.pix.height * 3;
       break;
     case V4L2_PIX_FMT_YUV420:
+    case V4L2_PIX_FMT_YVU420:
       dest_needed =
 	my_dest_fmt.fmt.pix.width * my_dest_fmt.fmt.pix.height * 3 / 2;
       temp_needed =
@@ -676,7 +792,7 @@ int v4lconvert_convert(struct v4lconvert_data *data,
   /* convert_pixfmt -> rotate -> crop, all steps are optional */
   if (convert && (rotate || crop)) {
     convert_dest = v4lconvert_alloc_buffer(data, temp_needed,
-                     &data->convert_buf, &data->convert_buf_size);
+		     &data->convert_buf, &data->convert_buf_size);
     if (!convert_dest)
       return -1;
 
@@ -685,7 +801,7 @@ int v4lconvert_convert(struct v4lconvert_data *data,
 
   if (rotate && crop) {
     rotate_dest = v4lconvert_alloc_buffer(data, temp_needed,
-                    &data->rotate_buf, &data->rotate_buf_size);
+		    &data->rotate_buf, &data->rotate_buf_size);
     if (!rotate_dest)
       return -1;
 
@@ -693,11 +809,9 @@ int v4lconvert_convert(struct v4lconvert_data *data,
   }
 
   if (convert) {
-    res = v4lconvert_convert_pixfmt(data, my_src_fmt.fmt.pix.pixelformat,
-			      my_dest_fmt.fmt.pix.pixelformat,
-			      my_src_fmt.fmt.pix.width,
-			      my_src_fmt.fmt.pix.height,
-			      src, src_size, convert_dest);
+    res = v4lconvert_convert_pixfmt(data, src, src_size, convert_dest,
+				    &my_src_fmt,
+				    my_dest_fmt.fmt.pix.pixelformat);
     if (res)
       return res;
 
@@ -722,7 +836,7 @@ const char *v4lconvert_get_error_message(struct v4lconvert_data *data)
 }
 
 static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
-  unsigned int pixelformat)
+  unsigned int pixelformat, int index)
 {
   int i, j, match;
   struct v4l2_frmsizeenum frmsize = { .pixel_format = pixelformat };
@@ -734,7 +848,7 @@ static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
 
     /* We got a framesize, check we don't have the same one already */
     match = 0;
-    for (j = 0; j < data->no_framesizes && !match; j++) {
+    for (j = 0; j < data->no_framesizes; j++) {
       if (frmsize.type != data->framesizes[j].type)
 	continue;
 
@@ -751,6 +865,8 @@ static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
 	    match = 1;
 	  break;
       }
+      if (match)
+	break;
     }
     /* Add this framesize if it is not already in our list */
     if (!match) {
@@ -760,6 +876,9 @@ static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
 	return;
       }
       data->framesizes[data->no_framesizes].type = frmsize.type;
+      /* We use the pixel_format member to store a bitmask of all
+	 supported src_formats which can do this size */
+      data->framesizes[data->no_framesizes].pixel_format = 1 << index;
       switch(frmsize.type) {
 	case V4L2_FRMSIZE_TYPE_DISCRETE:
 	  data->framesizes[data->no_framesizes].discrete = frmsize.discrete;
@@ -771,6 +890,8 @@ static void v4lconvert_get_framesizes(struct v4lconvert_data *data,
       }
       data->no_framesizes++;
     }
+    else
+      data->framesizes[j].pixel_format |= 1 << index;
   }
 }
 
@@ -805,8 +926,12 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
   int res;
   struct v4l2_format src_fmt, dest_fmt;
 
-  if (!v4lconvert_supported_dst_format(frmival->pixel_format))
-    return syscall(SYS_ioctl, data->fd, VIDIOC_ENUM_FRAMEINTERVALS, frmival);
+  if (!v4lconvert_supported_dst_format(frmival->pixel_format)) {
+    res = syscall(SYS_ioctl, data->fd, VIDIOC_ENUM_FRAMEINTERVALS, frmival);
+    if (res)
+      V4LCONVERT_ERR("%s\n", strerror(errno));
+    return res;
+  }
 
   /* Check which format we will be using to convert to frmival->pixel_format */
   memset(&dest_fmt, 0, sizeof(dest_fmt));
@@ -814,13 +939,30 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
   dest_fmt.fmt.pix.pixelformat = frmival->pixel_format;
   dest_fmt.fmt.pix.width = frmival->width;
   dest_fmt.fmt.pix.height = frmival->height;
-  if ((res = v4lconvert_try_format(data, &dest_fmt, &src_fmt)))
+  if ((res = v4lconvert_try_format(data, &dest_fmt, &src_fmt))) {
+    if (res)
+      V4LCONVERT_ERR("trying format: %s\n", strerror(errno));
     return res;
+  }
 
   /* Check the requested format is supported exactly as requested */
   if (dest_fmt.fmt.pix.pixelformat != frmival->pixel_format ||
       dest_fmt.fmt.pix.width  != frmival->width ||
       dest_fmt.fmt.pix.height != frmival->height) {
+    int frmival_pixformat = frmival->pixel_format;
+    int dest_pixformat = dest_fmt.fmt.pix.pixelformat;
+    V4LCONVERT_ERR("Could not find matching framesize for: %c%c%c%c %dx%d "
+            "closest match: %c%c%c%c %dx%d\n",
+	    frmival_pixformat & 0xff,
+	    (frmival_pixformat >> 8) & 0xff,
+	    (frmival_pixformat >> 16) & 0xff,
+	    frmival_pixformat >> 24,
+	    frmival->width, frmival->height,
+	    dest_pixformat & 0xff,
+	    (dest_pixformat >> 8) & 0xff,
+	    (dest_pixformat >> 16) & 0xff,
+	    dest_pixformat >> 24,
+	    dest_fmt.fmt.pix.width , dest_fmt.fmt.pix.height);
     errno = EINVAL;
     return -1;
   }
@@ -830,6 +972,23 @@ int v4lconvert_enum_frameintervals(struct v4lconvert_data *data,
   frmival->width = src_fmt.fmt.pix.width;
   frmival->height = src_fmt.fmt.pix.height;
   res = syscall(SYS_ioctl, data->fd, VIDIOC_ENUM_FRAMEINTERVALS, frmival);
+  if (res) {
+    int dest_pixfmt = dest_fmt.fmt.pix.pixelformat;
+    int src_pixfmt  = src_fmt.fmt.pix.pixelformat;
+    V4LCONVERT_ERR("Could not enum frameival index: %d for: %c%c%c%c %dx%d "
+            "using src: %c%c%c%c %dx%d, error: %s\n",
+            frmival->index,
+	    dest_pixfmt & 0xff,
+	    (dest_pixfmt >> 8) & 0xff,
+	    (dest_pixfmt >> 16) & 0xff,
+	    dest_pixfmt >> 24,
+	    dest_fmt.fmt.pix.width , dest_fmt.fmt.pix.height,
+	    src_pixfmt & 0xff,
+	    (src_pixfmt >> 8) & 0xff,
+	    (src_pixfmt >> 16) & 0xff,
+	    src_pixfmt >> 24,
+	    src_fmt.fmt.pix.width, src_fmt.fmt.pix.height, strerror(errno));
+  }
 
   /* Restore the requested format in the frmival struct */
   frmival->pixel_format = dest_fmt.fmt.pix.pixelformat;
